!----------------------------------------------------------------------
!  Construct boundary conditions for FLIP region
!
!  Author: Jiannan Tu
!  Date:   March 21, 2003
!----------------------------------------------------------------------
subroutine flip_ub(nst)
    use inparm
    use gskparm

    real(8) denO(nc),velO(nc),tprO(nc),tppO(nc),qprO(nc),qppO(nc)
    real(8) denH(nc),velH(nc),tprH(nc),tppH(nc),qprH(nc),qppH(nc)
    real(8) denHe(nc),velHe(nc)
    real(8) Re,ds

    real(8) Fb_denOn,Fb_denHn,Fb_ddenOn,Fb_ddenHn,Fb_denHen,Fb_ddenHen
    real(8) Fb_velOn,Fb_velHn,Fb_dvelOn,Fb_dvelHn,Fb_velHen,Fb_dvelHen
    real(8) Fb_tempOn,Fb_tempHn,Fb_dtempOn,Fb_dtempHn
    real(8) Fb_Ten,Fb_dTen,Fb_fhOn,Fb_fhHn,Fb_tempO1,Fb_tempH1
    real(8) Fb_denOs,Fb_denHs,Fb_ddenOs,Fb_ddenHs,Fb_denHes,Fb_ddenHes
    real(8) Fb_velOs,Fb_velHs,Fb_dvelOs,Fb_dvelHs,Fb_velHes,Fb_dvelHes
    real(8) Fb_tempOs,Fb_tempHs,Fb_dtempOs,Fb_dtempHs
    real(8) Fb_Tes,Fb_dTes,Fb_fhOs,Fb_fhHs

    integer nst,LNBLNK
    integer rank,ierr,status
    common /mpi/rank,ierr,status


!  pass back data of bottom cell to FLIP by Y.J.SU on 10/6/95
    common/denbackn/Fb_denOn,Fb_denHn,Fb_denHen,Fb_ddenOn,Fb_ddenHn,Fb_ddenHen
    common/velbackn/Fb_velOn,Fb_velHn,Fb_velHen,Fb_dvelOn,Fb_dvelHn,Fb_dvelHen
    common/tempbackn/Fb_tempOn,Fb_tempHn,Fb_dtempOn,Fb_dtempHn,Fb_Ten,Fb_dTen
    common /heatbackn/ Fb_fhOn,Fb_fhHn
!south hemisphere
    common/denbacks/Fb_denOs,Fb_denHs,Fb_denHes,Fb_ddenOs,Fb_ddenHs,Fb_ddenHes
    common/velbacks/Fb_velOs,Fb_velHs,Fb_velHes,Fb_dvelOs,Fb_dvelHs,Fb_dvelHes
    common/tempbacks/Fb_tempOs,Fb_tempHs,Fb_dtempOs,Fb_dtempHs,Fb_Tes,Fb_dTes
    common /heatbacks/ Fb_fhOs,Fb_fhHs

    data Re/6371.2d5/  ! Here Re in cm

! Data of 1100 km cell to pass back to FLIP Y.J.Su on 10/6/95
    ds=(sg(jgfn+1)-sg(jgfn))*Re

    Fb_denOn=den_O(jgfn)
    Fb_denHn=den_H(jgfn)
    Fb_denHen=den_He(jgfn)

    Fb_ddenOn=(den_O(jgfn+1)-Fb_denOn)/ds
    Fb_ddenHn=(den_H(jgfn+1)-Fb_denHn)/ds
    Fb_ddenHen=(den_He(jgfn+1)-Fb_denHen)/ds

    Fb_velOn=100.0d0*vel_O(jgfn)
    Fb_velHn=100.0d0*vel_H(jgfn)
    Fb_velHen=100.0d0*vel_He(jgfn)

    Fb_dvelOn=(100.0d0*vel_O(jgfn+1)-Fb_velOn)/ds
    Fb_dvelHn=(100.0d0*vel_H(jgfn+1)-Fb_velHn)/ds
    Fb_dvelHen=(100.0d0*vel_He(jgfn+1)-Fb_velHen)/ds

    Fb_tempOn=(tpr_O(jgfn)+2.0d0*tpp_O(jgfn))/3.0d0
    Fb_tempHn=(tpr_H(jgfn)+2.0d0*tpp_H(jgfn))/3.0d0

    Fb_tempO1=(tpr_O(jgfn+1)+2.0d0*tpp_O(jgfn+1))/3.0d0
    Fb_tempH1=(tpr_H(jgfn+1)+2.0d0*tpp_H(jgfn+1))/3.0d0
    Fb_dtempOn=(Fb_tempO1-Fb_tempOn)/ds
    Fb_dtempHn=(Fb_tempH1-Fb_tempHn)/ds

    Fb_ten=(tpr_e(jgfn)+2.0d0*tpp_e(jgfn))/3.0d0
    Fb_dten=((tpr_e(jgfn+1)+2.0d0*tpp_e(jgfn+1))/3.0d0-Fb_ten)/ds

!  heat flow (qprO, qppO etc.) in J m/s. Fb_fhO, Fb_fhH in ergs/(cm^2 s)
    Fb_fhOn=1.0d9*Fb_denOn*(qpr_O(jgfn)+2.0d0*qpp_O(jgfn))/3.0d0
    Fb_fhHn=1.0d9*Fb_denHn*(qpr_H(jgfn)+2.0d0*qpp_H(jgfn))/3.0d0

    ds=(sg(jgfs)-sg(jgfs-1))*Re

    Fb_denOs=den_O(jgfs)
    Fb_denHs=den_H(jgfs)
    Fb_denHes=den_He(jgfs)

    Fb_ddenOs=(Fb_denOs-den_O(jgfs-1))/ds
    Fb_ddenHs=(Fb_denHs-den_H(jgfs-1))/ds
    Fb_ddenHes=(Fb_denHes-den_He(jgfs-1))/ds

    Fb_velOs=100.0d0*vel_O(jgfs)
    Fb_velHs=100.0d0*vel_H(jgfs)
    Fb_velHes=100.0d0*vel_He(jgfs)

    Fb_dvelOs=(Fb_velOs-100.0d0*vel_O(jgfs-1))/ds
    Fb_dvelHs=(Fb_velHs-100.0d0*vel_H(jgfs-1))/ds
    Fb_dvelHes=(Fb_velHes-100.0d0*vel_He(jgfs-1))/ds

    Fb_tempOs=(tpr_O(jgfs)+2.0d0*tpp_O(jgfs))/3.0d0
    Fb_tempHs=(tpr_H(jgfs)+2.0d0*tpp_H(jgfs))/3.0d0

    Fb_tempO1=(tpr_O(jgfs-1)+2.0d0*tpp_O(jgfs-1))/3.0d0
    Fb_tempH1=(tpr_H(jgfs-1)+2.0d0*tpp_H(jgfs-1))/3.0d0
    Fb_dtempOs=(Fb_tempOs-Fb_tempO1)/ds
    Fb_dtempHs=(Fb_tempHs-Fb_tempH1)/ds

    Fb_tes=(tpr_e(jgfs)+2.0d0*tpp_e(jgfs))/3.0d0
    Fb_dtes=(Fb_tes-(tpr_e(jgfs-1)+2.0d0*tpp_e(jgfs-1))/3.0d0)/ds

!  heat flow (qprO, qppO etc.) in J m/s. Fb_fhO, Fb_fhH in ergs/(cm^2 s)
    Fb_fhOs=1.0d9*Fb_denOs*(qpr_O(jgfs)+2.0d0*qpp_O(jgfs))/3.0d0
    Fb_fhHs=1.0d9*Fb_denHs*(qpr_H(jgfs)+2.0d0*qpp_H(jgfs))/3.0d0


	   if(mod(nst,iout)==0.and.rank.eq.0) then
           open(11,file='/home/wangy/DyFk/INITIAL/BCKFLIP.dat', status='unknown')
           write(11,*) Fb_denOn,Fb_denHn,Fb_ddenOn,Fb_ddenHn,Fb_denHen, &
                       Fb_ddenHen,Fb_velOn,Fb_velHn,Fb_dvelOn,Fb_dvelHn, &
                       Fb_velHen,Fb_dvelHen,Fb_tempOn,Fb_tempHn,Fb_dtempOn, &
                       Fb_dtempHn,Fb_Ten,Fb_dTen,Fb_fhOn,Fb_fhHn,Fb_denOs, &
                       Fb_denHs,Fb_ddenOs,Fb_ddenHs,Fb_denHes,Fb_ddenHes, &
                       Fb_velOs,Fb_velHs,Fb_dvelOs,Fb_dvelHs,Fb_velHes, &
                       Fb_dvelHes,Fb_tempOs,Fb_tempHs,Fb_dtempOs, &
                       Fb_dtempHs,Fb_Tes,Fb_dTes,Fb_fhOs,Fb_fhHs
           close(11)
       endif


end subroutine flip_ub
