!***********************************************************************
!***********************************************************************
!   SUBROUTINE LGRG(X,Y,N,T,Z)
!    Eight-point Lagrange's interpolation and extrpolation
!    Given arrays X and Y, each of length N, and given a value of T,
!    this routine returns a value Z. The interpolation or extrapolation
!    is performed using lagrange's formula.
!***********************************************************************
SUBROUTINE LGRG(X,Y,N,T,Z)
    INTEGER N,I,J,K,M
    REAL(8) X(N),Y(N),T,Z,S

    Z=0.D0
    K=N-4
    M=N

    I=1
    DO WHILE (X(I).LT.T)
      I=I+1
      IF(I.GT.N) EXIT
    ENDDO

    K=I-4
    IF(K.LT.1) K=1
    M=I+3
    IF(M.GT.N) M=N

    DO I=K,M
      S=1.0d0
      DO J=K,M
        IF(J.NE.I) S=S*(T-X(J))/(X(I)-X(J))
      ENDDO
      Z=Z+S*Y(I)
    ENDDO

END SUBROUTINE
