!-----------------------------------------------------------------------
!  Move oxygen simulation particles
!
!  Rewritten and commented by Jiannan Tu, Mar 19, 2003. JNT 1/25/10 Rev
!-----------------------------------------------------------------------
subroutine move_Oxygen(efield,dppo)
    use gskparm

    real(8), dimension(nc) :: efield,dppo
    real(8) :: am_O,s,vs,vp,qmrO,ran2

    integer :: i,j,nnew

    integer rank,ierr,status
    common /mpi/rank,ierr,status


! qmrO=e/mO/Re
    data am_o/2.65676233d-26/,qmrO/0.946548309d0/

!  Consider particles newly produced
!------------------------------------------------------------------------
!  P_O(i): O+ from photoionization in 4 seconds.
!------------------------------------------------------------------------
  
        ! newly produced O+ by photoionization; having initial
        ! Maxwell distribution of oxygen
        nnew=P_O(rank+1)+ran2(iseed)
        if (nnew.gt.0) call neutral_produced(rank+1,imax_O,vt_O,nnew,Oxygen)

        ! newly produced O+ by H+ + O reaction; having initial
        ! Maxwell distribution of hydrogen
        nnew=P_O_H(rank+1)+ran2(iseed)
        if (nnew.gt.0) call neutral_produced(rank+1,imax_O,vt_H,nnew,Oxygen)

    P_O_H=0.0d0

    !call num_in_cell(numO,Oxygen,imax_O)
    do j=1,imax_O
        s=Oxygen(j,1)

        ! ignore the ptcl that is out of range
        if(s.lt.sg(1).or.s.ge.sg(num+1)) cycle

        vs=Oxygen(j,2)
        vp=Oxygen(j,3)

        ! heating by electron.
!        call addheat(i,s,vs,am_o,dtg,den_e,tpr_e,den_O,tpr_O,0)
!        call addheat(i,s,vp,am_o,dtg,den_e,tpp_e,den_O,tpp_O,1)

        !X.Y.WU03/05/98 wave-induced heating
        if(dppo(rank+1).gt.0.0d0) call waveheat(rank+1,s,vp,dppo,dtg,am_o)

        !if(alt(i).lt.5000.0d0) then
            ! collide with neutrals; not affect location
            !call coll_neutral(i,s,vs,vp,dtg)
!            call coll_elec(s,vs,vp)

            ! react with neutrals. If occurs, O+ removed (set s=999)
            !call react_neutral(i,s,vs,vp,dtg)
        !endif

        ! advance particle subjeted to various forces
        if(s.ge.sg(rank+1).and.s.lt.sg(rank+2)) then
            call mover(rank+1,efield,s,vs,vp,dtg,qmrO)
        endif

        Oxygen(j,1)=s
        Oxygen(j,2)=vs
        Oxygen(j,3)=vp

    enddo

    ! compact O+ index (bubble those with s=999.0 to the top)
    call compind1(imax_O,Oxygen)

end subroutine move_Oxygen
