!----------------------------------------------------------------------
!  Calculate reduced velocity space distribution:
!      altitude-velocity variation of particle distribution
!
!    Input:  imax  - total number of sim particles
!            rep   - number of real(8) particles in one sim particle
!            vsmin - minimum velocity (para or perp) for resulting
!                    distribution (in km/s)
!            dvs   - parallel velocity interval (in km/s)
!            dvp   - perp velocity interval (in km/s)
!            OHgen - array of location (in Re), para and perp velocity
!                    (in Re/s) of sim particles
!            nss   - number of grid in parallel velocity axis: = noss
!                    for O+; = nhss for H+
!            nsp   - number of grid in perpendicular velocity axis
!
!    Output: vss_s - altitude parallel velocity distribution
!            vps_s - altitude perp velocity distribution
!
!  Jiannan Tu, Feb 13, 2003
!----------------------------------------------------------------------
subroutine reduced_vdistr(imax,vsmin,dvs,dvp,OHgen,vss_s,vps_s,nss,nsp)

      use gskparm

      integer nss,nsp

      real(8) OHgen(np,3),vss_s(nss,100),vps_s(nsp,100)
      real(8) vsmin,dvs,dvp,s,ds,vs,vp,smin,vsminr,dvpr,dvsr
      real(8) aii,bii,dai1,dai2,Re,upaii,upbii
      integer imax,ii,jj,it,i

      data Re /6371.2/ ! Here Re in km,because vsmin, dvs & dvp in km/s

      upaii=float(nss)+1.5d0
      upbii=float(nsp)+1.5d0
      ds=(sg(num+1)-sg(1))/100.0d0
      smin=sg(1)

      vsminr=vsmin/Re
      dvsr=dvs/Re
      dvpr=dvp/Re

      do i=1,imax
        s=OHgen(i,1)

        if(s.lt.sg(1).or.s.ge.sg(num+1)) cycle

        vs=OHgen(i,2)
        vp=OHgen(i,3)

        aii=(vs-vsminr)/dvsr+1.0d0
        bii=vp/dvpr+1.0d0
        jj=int((s-smin)/ds+1.0d0)

        if(aii.gt.0.0d0.and.aii.le.upaii) then
          ii=int(aii)

          dai1=aii-float(ii)-0.5d0
          if (dai1.lt.0.0d0) then
            it=ii-1
            dai1=abs(dai1)
          else
            it=ii+1
          endif
          dai2=1.0d0-dai1

          if(ii.ge.1.and.ii.le.nss.and.jj.ge.1.and.jj.le.100) &
            vss_s(ii,jj)=vss_s(ii,jj)+dai2
          if(it.ge.1.and.it.le.nss.and.jj.ge.1.and.jj.le.100) &
            vss_s(it,jj)=vss_s(it,jj)+dai1

          if(bii.gt.0.0d0.and.bii.le.upbii) then
            ii=int(bii)

            dai1=bii-float(ii)-0.5d0
            if(dai1.lt.0.0d0) then
              it=ii-1
              dai1=abs(dai1)
            else
              it=ii+1
            endif
            dai2=1.0d0-dai1

            if(ii.ge.1.and.ii.le.nsp.and.jj.ge.1.and.jj.le.100) &
              vps_s(ii,jj)=vps_s(ii,jj)+dai2
            if(it.ge.1.and.it.le.nsp.and.jj.ge.1.and.jj.le.100) &
              vps_s(it,jj)=vps_s(it,jj)+dai1
          endif
        endif
      enddo

end subroutine reduced_vdistr

!---------------------------------------------------------
!  Randomly set values of elemnets of array vxyz(1:10000)
!  vxyz is used in calculation of O+ ion collision and
!  reaction with neutrals
!---------------------------------------------------------
subroutine set_vxyz()
      use gskparm

      real(8) pi2,the2,vp,ran2
      integer i

      data Pi2/6.283185308/

      do i=1,10000
         vxyz(i,3)=sqrt(-log(1.0d0-ran2(iseed)))*cos(Pi2*ran2(iseed))
         the2=Pi2*ran2(iseed)
         vp=sqrt(-log(1.0d0-ran2(iseed)))
         vxyz(i,1)=vp*cos(the2)
         vxyz(i,2)=vp*sin(the2)
      enddo

end subroutine set_vxyz
