!-----------------------------------------------------------------------
!  This is a time dependent generalized semi-kinetic model for simulating
!  plasma transport along a closed, drifitng flux tube. The model 
!  advances O+ and H+ simulation particles subjected to gravity, electric
!  field, mirror, and centrfigual forces. Also included are O+
!  ion-neutral collisions, ion binary Coulobm collisions and heat
!  transfer between electron and ions. 
!
!  This program is written by Jiannan Tu based on previous gsk codes by
!  C. Wing Ho, David G. Brown, Yi-Jiun Su, Garry M. Estep, XiangYang Wu
!  and Jiannan Tu.
!
!  Author: Jiannan Tu
!  Created: August 1, 2003; Modified 4/30/2010
!-----------------------------------------------------------------------
subroutine tgsk(nst,jmax,Z,ZT,PCO)
      USE inparm
      use gskparm
      include 'mpif.h'

      real(8) Te(nc),Tn(nc) !,v_vol(100)

      real(8) poten(nc),efield(nc),curt(nc),dppo(nc),dpph(nc),dpphe(nc)

      real(8) Z(401),PCO,ZT

!      real(8) a,r_a,v_dr,v_con,r_b,b
      real(8) Req,sigma,pfreq,ggtemp,we0,curt0,DAR
      real(8) M_O,M_H,M_N2,M_O2,M_He,ak,Re,pi,degrad

      integer IDAY,i,j,err0,nume,num_s(3,num)
      integer jbn,jbs,nst,jmax,LNBLNK
      

      real AP(7),DEC,ETRAN,BLON,F107,F107A,SEC,GLATD,GLOND,SAT
      real(8) GL(401) !,energy

      real(4) UTSEC,atomD(8),atomT(2)
      logical first

      SAVE

      real(8) denO,velO,tprO,tppO,qprO,qppO
      real(8) denH,velH,tprH,tppH,qprH,qppH
      real(8) denHe,velHe,tprHe,tppHe
      real(8) qprHe,qppHe

      real(8) vss_O(noss,100),vps_O(nosp,100),vsvp_O(nossm,nospm)
      real(8) vss_H(nhss,100),vps_H(nhsp,100),vsvp_H(nhssm,nhspm)
      real(8) vss_He(nhess,100),vps_He(nhesp,100),vsvp_He(nhessm,nhespm)

      real(8) vss_O_tmp(noss,100),vps_O_tmp(nosp,100),vsvp_O_tmp(nossm,nospm)
      real(8) vss_H_tmp(nhss,100),vps_H_tmp(nhsp,100),vsvp_H_tmp(nhssm,nhspm)
      real(8) vss_He_tmp(nhess,100),vps_He_tmp(nhesp,100),vsvp_He_tmp(nhessm,nhespm)

      real(8) dummy1(np1,3),dummy2(np1,3),dummy4(np1,3),dummy5(np1,3)
      real(8) dummyp1(np1,3),dummyp2(np1,3),dummyp4(np1,3),dummyp5(np1,3)

!   pass mpi parameter
      integer rank,ierr,status
      common /mpi/rank,ierr,status

!  pass geophysical condition from FLIP
      COMMON/MSIS/AP,DEC,ETRAN,BLON,F107,F107A,IDAY,SEC,GL

      data M_O/2.65676233d-26/,M_H/1.67382304d-27/,M_N2/4.65174685d-26/
      data M_O2/5.3135247d-26/,Re/6371.2e3/,ak/1.38065d-23/
      data M_He/6.69529216d-27/,first/.true./
      data sigma/1.5d-15/
     
      print*, 'rank=',rank,'finish defining variables in GSK'

!**************** Broadcast parameters obtained from FLIP **********************
 
      call MPI_BCAST(nst,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)           
      call MPI_BCAST(jmax,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr) 
      !print *,'rank=',rank,'jmax=',jmax,'nst=',nst
      !pause
      call MPI_BCAST(Z,401,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(ZT,1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
      !print*,'Z(1)=',Z(1),'Z(10)=',Z(10),'Z(100)=',Z(100),'Z(400)=',Z(400)
      !print*,'rank=',rank,'&ZT=',ZT,'MPI_COMM_WORLD=',MPI_COMM_WORLD
      !pause
      call MPI_BCAST(PCO,1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(AP,7,MPI_REAL,0,MPI_COMM_WORLD,ierr)
      !print *, 'rank=',rank,'PCO=',PCO
      !print *, 'rank=',rank,'AP=',AP
      !pause
      call MPI_BCAST(DEC,1,MPI_REAL,0,MPI_COMM_WORLD,ierr)
      !print *,'rank=',rank,'PCO=',PCO,'DEC=',DEC
      !pause
      call MPI_BCAST(ETRAN,1,MPI_REAL,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(BLON,1,MPI_REAL,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(F107,1,MPI_REAL,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(F107A,1,MPI_REAL,0,MPI_COMM_WORLD,ierr)
      !print *,'rank=',rank,'ETRAN=',ETRAN,'&BLON=',BLON,'&F107A=',F107A,'&F107=',F107
      CALL MPI_BCAST(SEC,1,MPI_REAL,0,MPI_COMM_WORLD,ierr)
      CALL MPI_BCAST(GLATD,1,MPI_REAL,0,MPI_COMM_WORLD,ierr)
      CALL MPI_BCAST(GLOND,1,MPI_REAL,0,MPI_COMM_WORLD,ierr)
      CALL MPI_BCAST(SAT,1,MPI_REAL,0,MPI_COMM_WORLD,ierr)
      CALL MPI_BCAST(GL,401,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
      !print *,'rank=',rank,'&SEC=',SEC,'&GLATD=',GLATD,'&GLOND=',GLOND,'&SAT=',SAT,'&GL=',GL
      print *, 'rank=', rank, 'finish first broadcast parameters from Flip in GSK here'

      err0=0
     
      UTSEC=mod(int(SEC), 86400)

! Req: L-shell. PCO varying if flux tube drifts
      Req=PCO

!  zero elements of various arrays
      !call init_array2(denO,velO,tprO,tppO,qprO,qppO,denH,velH,tprH,&
      !        tppH,qprH,qppH,denHe,velHe,tprHe,tppHe,qprHe,qppHe)
      call init_array3(vsvp_O_tmp,vss_O_tmp,vps_O_tmp,vsvp_H_tmp,vss_H_tmp,&
              vps_H_tmp,vsvp_He_tmp,vss_He_tmp,vps_He_tmp)

! determine grid indice for GSK bottom boundaries in FLIP zone
      do j=1,JMAX/2
        if(Z(j).ge.zb) then
          jbn=j-1
          exit
        endif
      enddo
      do j=JMAX,JMAX/2,-1
        if(Z(j).ge.zb) then
          jbs=j+1
          exit
        endif
      enddo

! calculate q values from N to S in GSK zone
      call qvalue(num,Req,Z,scalg,jbn,qval)

! calcualte grid parameters along the flux tube
      call calc_grid(nst,Req)

! determine grid indice for FLIP top boundaries in GSK zone
      do j=1,num/2+1
        if(alt(j).ge.zt) then
          jgfn=j-1
          exit
        endif
      enddo
      do j=num+1,num/2+1,-1
        if(alt(j).ge.zt) then
          jgfs=j+1
          exit
        endif
      enddo

      print *,'nst=', nst, 'rank=', rank, ' finish grid calculation'

      if(first.or.Drift.eq.'Yes') then
        read(46,*) we0,Veq,curt0 !Veq in m/s
        call MPI_BCAST(we0,1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
        call MPI_BCAST(Veq,1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
        call MPI_BCAST(curt0,1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
        Veq=Veq/Re    !convert to unit of Re/s
        Veq2=Veq*Veq
      endif

      print *, 'nst=', nst, 'rank=', rank, 'finished second BCAST in GSK'

      if(first) then
         call init_array1()
         call init_array4()

         pi=acos(-1.0d0)
         degrad=pi/180.0d0

        !Ionization frequency (Schunk & Nagy, Ionospheres, Chap 9, p. 245,
        !Table 9.2): = 2.44d-7/s in solar min;  = 6.346d-7/s in solar max 
         if(F107.gt.130) pfreq=6.346d-7
         if(F107.le.130) pfreq=2.44d-7
       
       ! read collision cross sections for O+ (with O, N2 and O2)
         open(11,file='/home/wangy/DyFk/SETUP/ON2O2.DAT',status='old')
         do i=1,326
            read(11,'(f5.2,2f7.3)') ggtemp,gg_sig(1,i),gg_sig(2,i)
         enddo
         close(11)
		       
!	 call MPI_BARRIER(MPI_COMM_WORLD,ierr)

         call MPI_BCAST(ggtemp,1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)

         call MPI_BCAST(gg_sig,2*326,MPI_REAL8,0,MPI_COMM_WORLD,ierr)

         print *, 'nst=', nst, 'rank=', rank, 'finished third BCAST in GSK'


        ! input particles from files
         call read_ptcl(err0,rank)     !need to change for every rank
         print *, 'nst=', nst, 'rank=', rank, 'finished reading particles from file in GSK'
!         pause

!        call MPI_BARRIER(MPI_COMM_WORLD,ierr)


        select case (err0)
          case (0)  !successfully to input ptcls          
            ! read electron moments profile
			
            open(11,file='/home/wangy/DyFk/INITIAL/moments.dat',status='old')
            read(11,'(A)')
            do i=1,num+1
                read(11,*) den_e(i),den_e(i),Te(i),tpr_e(i),tpp_e(i),vel_e(i),efield(i),&
                           den_O(i),vel_O(i),tpr_O(i),tpp_O(i),&
                           den_H(i),vel_H(i),tpr_H(i),tpp_H(i), &
                           den_He(i),vel_He(i),tpr_He(i),tpp_He(i), &
                           qpr_O(i),qpp_O(i),qpr_H(i), qpp_H(i),&
                           qpr_He(i),qpp_He(i),qpr_e(i),qpp_e(i)
            enddo
            close(11)
        
            call MPI_BCAST(den_e,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
	    call MPI_BCAST(Te,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
	    call MPI_BCAST(tpr_e,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
	    call MPI_BCAST(tpp_e,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
	    call MPI_BCAST(vel_e,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
	    call MPI_BCAST(efield,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
            call MPI_BCAST(den_O,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
            call MPI_BCAST(vel_O,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
	    call MPI_BCAST(tpr_O,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
	    call MPI_BCAST(tpp_O,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
	    call MPI_BCAST(den_H,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
	    call MPI_BCAST(vel_H,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
	    call MPI_BCAST(tpr_H,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
	    call MPI_BCAST(tpp_H,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
	    call MPI_BCAST(den_He,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
	    call MPI_BCAST(vel_He,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
	    call MPI_BCAST(tpr_He,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
	    call MPI_BCAST(tpp_He,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
	    call MPI_BCAST(qpr_O,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
	    call MPI_BCAST(qpp_O,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
	    call MPI_BCAST(qpr_H,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
	    call MPI_BCAST(qpp_H,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
	    call MPI_BCAST(qpr_He,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
	    call MPI_BCAST(qpp_He,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
	    call MPI_BCAST(qpr_e,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
	    call MPI_BCAST(qpp_e,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)

            print *, 'nst=', nst, 'rank=', rank, 'finished fourth BCAST in GSK'


          case (-1) !fimax_Oail to input ptcls
            ! start a new GSK run --> initialize ptcl distributions
            ! get initial e-, O+, H+ & He+ density, velocity, and temperature
            ! distributions based on the distributions from FLIP solution
            call interpol(jbn-4,jbs+4)

	    call MPI_BCAST(den_e,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
	    call MPI_BCAST(vel_e,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)			
	    call MPI_BCAST(tpr_e,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)		

	    call MPI_BCAST(den_O,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
	    call MPI_BCAST(vel_O,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
	    call MPI_BCAST(tpr_O,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)

	    call MPI_BCAST(den_H,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
	    call MPI_BCAST(vel_H,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
	    call MPI_BCAST(tpr_H,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
			
	    call MPI_BCAST(den_He,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
	    call MPI_BCAST(vel_He,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
	    call MPI_BCAST(tpr_He,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)			

            print *, 'nst=', nst, 'rank=', rank, 'finished fourth* BCAST in GSK'

            tpp_e=tpr_e; tpp_O=tpr_O; tpp_H=tpr_H; tpp_He=tpr_He

        open(21,file='moments_flip.dat',status='unknown')
           do i=1,num
              write(21,*) i,sg(i),(GLL(i)+GLL(i+1))*57.3/2.0,den_O(i),vel_O(i),&
                           tpr_O(i),tpp_O(i),den_H(i),vel_H(i),tpr_H(i),tpp_H(i),&
                           den_He(i),vel_He(i),tpr_He(i),tpp_He(i),den_e(i),&
                           vel_e(i),tpr_e(i),tpp_e(i)
                    
           enddo
        close(21)


            ! load O+, H+ & He+ assuming isotropic & drifitng Maxwell distri
            call load_ptcl(den_O,ndO,0,rep_O,num_s)
            call load_ptcl(den_H,ndH,1,rep_H,num_s)
            call load_ptcl(den_He,ndHe,2,rep_He,num_s)

            imax_O=num_s(1,rank+1)
            imax_H=num_s(2,rank+1)
            imax_He=num_s(3,rank+1)

            print *, 'nst=', nst, 'rank=', rank, 'finished load ptcl in GSK'
!            print *, 'nst,rank,imax_O,imax_H,imax_He:',nst,rank,imax_O,imax_H,imax_He

            call load_ptcl_mp(vel_O(rank+1),tpp_O(rank+1),tpr_O(rank+1),imax_O,0,Oxygen)  
            call load_ptcl_mp(vel_H(rank+1),tpp_H(rank+1),tpr_H(rank+1),imax_H,1,Hydrog)
            call load_ptcl_mp(vel_He(rank+1),tpp_He(rank+1),tpr_He(rank+1),imax_He,2,Helium) 

            print *, 'nst=', nst, 'rank=', rank, 'finished load ptcl_mp in GSK'
           
        end select
         ! initial electric field       
         call efdfind(efield,poten)

        ! make sure no more initialization
        first=.false.
      endif

!**************************** Calculate moments at current time step *********************
        call moments_s(Oxygen,imax_O,rep_O,denO,velO,tprO,tppO,qprO,qppO,0)
        print *, 'nst=', nst, 'rank=', rank, 'finished moments_s O+ in GSK'

        call moments_s(Hydrog,imax_H,rep_H,denH,velH,tprH,tppH,qprH,qppH,1)
        print *, 'nst=', nst, 'rank=', rank, 'finished moments_s H+ in GSK'

        call moments_s(Helium,imax_He,rep_He,denHe,velHe,tprHe,tppHe,qprHe,qppHe,2)
        print *, 'nst=', nst, 'rank=', rank, 'finished moments_s He+ in GSK'
        call MPI_BARRIER(MPI_COMM_WORLD,ierr)

!***************Gather the moments of all grids*******************************
        call MPI_ALLGATHER(denO,1,MPI_REAL8,den_O,1,MPI_REAL8,MPI_COMM_WORLD,ierr)
        call MPI_ALLGATHER(velO,1,MPI_REAL8,vel_O,1,MPI_REAL8,MPI_COMM_WORLD,ierr)
        call MPI_ALLGATHER(tprO,1,MPI_REAL8,tpr_O,1,MPI_REAL8,MPI_COMM_WORLD,ierr)
        call MPI_ALLGATHER(tppO,1,MPI_REAL8,tpp_O,1,MPI_REAL8,MPI_COMM_WORLD,ierr)
        call MPI_ALLGATHER(qprO,1,MPI_REAL8,qpr_O,1,MPI_REAL8,MPI_COMM_WORLD,ierr)
        call MPI_ALLGATHER(qppO,1,MPI_REAL8,qpp_O,1,MPI_REAL8,MPI_COMM_WORLD,ierr)

        call MPI_ALLGATHER(denH,1,MPI_REAL8,den_H,1,MPI_REAL8,MPI_COMM_WORLD,ierr)
        call MPI_ALLGATHER(velH,1,MPI_REAL8,vel_H,1,MPI_REAL8,MPI_COMM_WORLD,ierr)
        call MPI_ALLGATHER(tprH,1,MPI_REAL8,tpr_H,1,MPI_REAL8,MPI_COMM_WORLD,ierr)
        call MPI_ALLGATHER(tppH,1,MPI_REAL8,tpp_H,1,MPI_REAL8,MPI_COMM_WORLD,ierr)
        call MPI_ALLGATHER(qprH,1,MPI_REAL8,qpr_H,1,MPI_REAL8,MPI_COMM_WORLD,ierr)
        call MPI_ALLGATHER(qppH,1,MPI_REAL8,qpp_H,1,MPI_REAL8,MPI_COMM_WORLD,ierr)
  
        call MPI_ALLGATHER(denHe,1,MPI_REAL8,den_He,1,MPI_REAL8,MPI_COMM_WORLD,ierr)
        call MPI_ALLGATHER(velHe,1,MPI_REAL8,vel_He,1,MPI_REAL8,MPI_COMM_WORLD,ierr)
        call MPI_ALLGATHER(tprHe,1,MPI_REAL8,tpr_He,1,MPI_REAL8,MPI_COMM_WORLD,ierr)
        call MPI_ALLGATHER(tppHe,1,MPI_REAL8,tpp_He,1,MPI_REAL8,MPI_COMM_WORLD,ierr)
        call MPI_ALLGATHER(qprHe,1,MPI_REAL8,qpr_He,1,MPI_REAL8,MPI_COMM_WORLD,ierr)
        call MPI_ALLGATHER(qppHe,1,MPI_REAL8,qpp_He,1,MPI_REAL8,MPI_COMM_WORLD,ierr)

   
!****************** smooth moments (density, velocity & temperature)**************
        call smooth10(num)
        open(201,file='moments_ptcl.dat',status='unknown')
           do i=1,num
              write(201,*) i,sg(i),(GLL(i)+GLL(i+1))*57.3/2.0,den_O(i),vel_O(i),&
                           tpr_O(i),tpp_O(i),den_H(i),vel_H(i),tpr_H(i),tpp_H(i),&
                           den_He(i),vel_He(i),tpr_He(i),tpp_He(i),den_e(i),&
                           vel_e(i),tpr_e(i),tpp_e(i),efield(i)
                    
           enddo
        close(201)

      do i=1,num+1
        ! convert magnetic to geographic.
        call BTOG(0,BLON,SNGL(GLL(i)/degrad),GLATD,GLOND)

        ! Solar apparent time
        SAT=(UTSEC-ETRAN+43200.0d0)/3600.0d0+GLOND/15.0d0
        if(SAT.lt.0.0d0) SAT=SAT+24.0d0
        if(SAT.GT.24.0d0) SAT=SAT-24.0d0

        ! Call MSIS88 to get neutral density
        CALL GTS3(IDAY,UTSEC,SNGL(ALT(i)),GLATD,GLOND,SAT,F107A,F107,AP, &
                  48,atomD,atomT)

        atom_s(1,i)=atomD(2)  ! O density (cm^-3)
        atom_s(2,i)=atomD(7)  ! H density
        atom_s(3,i)=atomD(3)  ! N2 density
        atom_s(4,i)=atomD(4)  ! O2 density
        Tn(i)=atomT(2)

        if (atom_s(1,i).le.0.0d0) atom_s(1,i)=1.0d-10
        den_aO(i)=atom_s(1,i)

        ! O+ + H => H+ + O. See Banks/Kockarts, eq (21.2), p.192.
!        P_H(i)=1.5d-13*sqrt(8.0*ak*Tn(i)/(pi*M_H))*atom_s(2,i)* &
!               den_O(i)*vol(i)*dtg/rep_H

!--------
!  P_O(i): number of simulation O+ by photoionization at high altitudes
!  (Schunk/Nagy, p.245) in 4 seconds. P_O_H redount with O+ + H
!  reaction in subroutine move_Oxygen
!--------
        P_O(i)=1.6d6*pfreq*atom_s(1,i)*vol(i)*dtg/rep_O

        pol_N2(i)=-1.072038d-9*atom_s(3,i)
        atom_s(1,i)=10.0d0**16/(Pi*atom_s(1,i)*100.0d0*Re)
        atom_s(2,i)=atom_s(2,i)*sigma*100.0d0*Re
        atom_s(3,i)=atom_s(3,i)*Re*1.d-14
        atom_s(4,i)=atom_s(4,i)*Re*1.d-14

        ! Thermal velocity of neutral species
        vt_O(i)=sqrt(2.0d0*ak*Tn(i)/M_O)/Re
        vt_H(i)=sqrt(2.0d0*ak*Tn(i)/M_H)/Re
        vt_N2(i)=sqrt(2.0d0*ak*Tn(i)/M_N2)/Re
        vt_O2(i)=sqrt(2.0d0*ak*Tn(i)/M_O2)/Re
        vt_He(i)=sqrt(2.0d0*ak*Tn(i)/M_He)/Re
      enddo

      do i=1,num
        P_O(i)=0.5d0*(P_O(i)+P_O(i+1))
        P_H(i)=0.5d0*(P_H(i)+P_H(i+1))  !first need to cal P_H(i)
      enddo

      CALL MPI_BCAST(P_O,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
      CALL MPI_BCAST(P_H,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
      CALL MPI_BCAST(P_O_H,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
      CALL MPI_BCAST(P_He,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
      CALL MPI_BCAST(atom_s,4*nc,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
      CALL MPI_BCAST(den_aO,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
      CALL MPI_BCAST(vt_O,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
      CALL MPI_BCAST(vt_H,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
      CALL MPI_BCAST(vt_N2,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
      CALL MPI_BCAST(vt_O2,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
      CALL MPI_BCAST(vt_He,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
      CALL MPI_BCAST(pol_N2,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
      CALL MPI_BCAST(Tn,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)

      print *, 'nst=', nst, 'rank=', rank, 'finished fifth BCAST in GSK'

     !  Randomly set values of array vxyz(1:10000)
!      call set_vxyz()

     ! added by X.Y.Wu 03/05/98 wave heating rate profile
      if (nst.ge.WAVEON.and.nst.le.WAVEOF) then
        call waveh_rate(we0,dppo,dpph,dpphe)

        if(nst==WAVEON) write(16,*) WAVEON, 'Wave heating on'
        if(nst==WAVEOF) write(16,*) WAVEOF, 'Wave heating off'
      else
          dppo=-1.0d0
          dpph=-1.0d0
          dpphe=-1.0d0
      endif

     ! X.Y.Wu 11/11/99  read in hot plasma parameters
      if(nst==HOTPON) then
	 nume=num/2+1

         open(11,file='/home/wangy/DyFk/SETUP/hotplasma.dat', &
              status='old',form='formatted')
         read(11,*) den_p(nume),tpp_p(nume),tpr_p(nume),tpp_hote(nume), &
                    tpr_hote(nume)
         close(11)

         call MPI_BCAST(den_p,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
         call MPI_BCAST(tpp_p,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
	 call MPI_BCAST(tpr_p,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
	 call MPI_BCAST(tpp_hote,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
	 call MPI_BCAST(tpr_hote,num+1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)

         print *, 'nst=', nst, 'rank=', rank, 'finished BCAST in GSK'


        ! differitial anisotropy ratio (DAR)
         DAR=(tpp_p(nume)/tpr_p(nume))/(tpp_hote(nume)/tpr_hote(nume))
         write(16,*)'Including hot magnetospheric plasma',' DAR = ', DAR
      endif

! Current density (A/m^2) distribution along the flux tube
      if(nst.ge.CURON.and.nst.le.CUROF) then
         if(nst==CURON) write(16,*) CURON, 'FA current on'
         if(nst==CUROF) write(16,*) CUROF, 'FA current off'
      else
        curt0=0.0d0
      endif
      do i=1,num+1
         curt(i)=curt0*(rr(i)/rr(num+1))**3
      enddo


      OPEN(63,FILE='COURANT.LOG',STATUS='UNKNOWN')

! Number of O+, H+ & He+ injected upward in time period of dtg
     
       call num_inject(jbn,1)   !northern hemisphere

       call num_inject(jbs,2)   !southern hemisphere

       CALL MPI_SEND(bottom_upO,2,MPI_INTEGER,num-1,1,MPI_COMM_WORLD,ierr)
       CALL MPI_SEND(bottom_upH,2,MPI_INTEGER,num-1,2,MPI_COMM_WORLD,ierr)
       CALL MPI_SEND(bottom_upHe,2,MPI_INTEGER,num-1,3,MPI_COMM_WORLD,ierr)
       CALL MPI_SEND(factorO,2,MPI_REAL8,num-1,4,MPI_COMM_WORLD,ierr)
       CALL MPI_SEND(factorH,2,MPI_REAL8,num-1,5,MPI_COMM_WORLD,ierr)
       CALL MPI_SEND(factorHe,2,MPI_REAL8,num-1,6,MPI_COMM_WORLD,ierr)
       CALL MPI_SEND(fmaxO,2,MPI_REAL8,num-1,7,MPI_COMM_WORLD,ierr)
       CALL MPI_SEND(fmaxH,2,MPI_REAL8,num-1,8,MPI_COMM_WORLD,ierr)
       CALL MPI_SEND(fmaxHe,2,MPI_REAL8,num-1,9,MPI_COMM_WORLD,ierr)
       CALL MPI_SEND(FV_O,2,MPI_REAL8,num-1,10,MPI_COMM_WORLD,ierr)
       CALL MPI_SEND(FV_H,2,MPI_REAL8,num-1,11,MPI_COMM_WORLD,ierr)
       CALL MPI_SEND(FV_He,2,MPI_REAL8,num-1,12,MPI_COMM_WORLD,ierr)
       CALL MPI_SEND(vtprO,2,MPI_REAL8,num-1,13,MPI_COMM_WORLD,ierr)
       CALL MPI_SEND(VtppO,2,MPI_REAL8,num-1,14,MPI_COMM_WORLD,ierr)
       CALL MPI_SEND(vtprH,2,MPI_REAL8,num-1,15,MPI_COMM_WORLD,ierr)
       CALL MPI_SEND(VtppH,2,MPI_REAL8,num-1,16,MPI_COMM_WORLD,ierr)
       CALL MPI_SEND(vtprHe,2,MPI_REAL8,num-1,17,MPI_COMM_WORLD,ierr)
       CALL MPI_SEND(VtppHe,2,MPI_REAL8,num-1,18,MPI_COMM_WORLD,ierr)

       print *, 'nst=', nst, 'rank=', rank, 'finished SEND in GSK'

       call MPI_BCAST(FheatE,2,MPI_REAL8,0,MPI_COMM_WORLD,ierr)

       print *, 'nst=', nst, 'rank=', rank, 'finished sixth BCAST in GSK'

    do j=1,loopmax


      
!*********************************************************************************
!                                Start main loop
!*********************************************************************************

        
      call MPI_BARRIER(MPI_COMM_WORLD,ierr)  !synchronize

      !First conduct Coulomb collisions
      call coulomb_coll() !(energy)

      print *, 'nst=', nst, 'j=',j,'rank=', rank, 'finished collision in GSK'
      call MPI_BARRIER(MPI_COMM_WORLD,ierr)  !synchronize

!**************************** Calculate moments at current time step *********************
        call moments_s(Oxygen,imax_O,rep_O,denO,velO,tprO,tppO,qprO,qppO,0)
        print *, 'nst=', nst, 'rank=', rank, 'finished moments_s O+ in GSK'

        call moments_s(Hydrog,imax_H,rep_H,denH,velH,tprH,tppH,qprH,qppH,1)
        print *, 'nst=', nst, 'rank=', rank, 'finished moments_s H+ in GSK'

        call moments_s(Helium,imax_He,rep_He,denHe,velHe,tprHe,tppHe,qprHe,qppHe,2)
        print *, 'nst=', nst, 'rank=', rank, 'finished moments_s He+ in GSK'
        call MPI_BARRIER(MPI_COMM_WORLD,ierr)

!***************Gather the moments of all grids*******************************
        call MPI_ALLGATHER(denO,1,MPI_REAL8,den_O,1,MPI_REAL8,MPI_COMM_WORLD,ierr)
        call MPI_ALLGATHER(velO,1,MPI_REAL8,vel_O,1,MPI_REAL8,MPI_COMM_WORLD,ierr)
        call MPI_ALLGATHER(tprO,1,MPI_REAL8,tpr_O,1,MPI_REAL8,MPI_COMM_WORLD,ierr)
        call MPI_ALLGATHER(tppO,1,MPI_REAL8,tpp_O,1,MPI_REAL8,MPI_COMM_WORLD,ierr)
        call MPI_ALLGATHER(qprO,1,MPI_REAL8,qpr_O,1,MPI_REAL8,MPI_COMM_WORLD,ierr)
        call MPI_ALLGATHER(qppO,1,MPI_REAL8,qpp_O,1,MPI_REAL8,MPI_COMM_WORLD,ierr)

        call MPI_ALLGATHER(denH,1,MPI_REAL8,den_H,1,MPI_REAL8,MPI_COMM_WORLD,ierr)
        call MPI_ALLGATHER(velH,1,MPI_REAL8,vel_H,1,MPI_REAL8,MPI_COMM_WORLD,ierr)
        call MPI_ALLGATHER(tprH,1,MPI_REAL8,tpr_H,1,MPI_REAL8,MPI_COMM_WORLD,ierr)
        call MPI_ALLGATHER(tppH,1,MPI_REAL8,tpp_H,1,MPI_REAL8,MPI_COMM_WORLD,ierr)
        call MPI_ALLGATHER(qprH,1,MPI_REAL8,qpr_H,1,MPI_REAL8,MPI_COMM_WORLD,ierr)
        call MPI_ALLGATHER(qppH,1,MPI_REAL8,qpp_H,1,MPI_REAL8,MPI_COMM_WORLD,ierr)
  
        call MPI_ALLGATHER(denHe,1,MPI_REAL8,den_He,1,MPI_REAL8,MPI_COMM_WORLD,ierr)
        call MPI_ALLGATHER(velHe,1,MPI_REAL8,vel_He,1,MPI_REAL8,MPI_COMM_WORLD,ierr)
        call MPI_ALLGATHER(tprHe,1,MPI_REAL8,tpr_He,1,MPI_REAL8,MPI_COMM_WORLD,ierr)
        call MPI_ALLGATHER(tppHe,1,MPI_REAL8,tpp_He,1,MPI_REAL8,MPI_COMM_WORLD,ierr)
        call MPI_ALLGATHER(qprHe,1,MPI_REAL8,qpr_He,1,MPI_REAL8,MPI_COMM_WORLD,ierr)
        call MPI_ALLGATHER(qppHe,1,MPI_REAL8,qpp_He,1,MPI_REAL8,MPI_COMM_WORLD,ierr)

   
!****************** smooth moments (density, velocity & temperature)**************
        call smooth10(num)
        open(201,file='moments_coll.dat',status='unknown')
           do i=1,num
              write(201,*) i,sg(i),(GLL(i)+GLL(i+1))*57.3/2.0,den_O(i),vel_O(i),&
                           tpr_O(i),tpp_O(i),den_H(i),vel_H(i),tpr_H(i),tpp_H(i),&
                           den_He(i),vel_He(i),tpr_He(i),tpp_He(i),den_e(i),&
                           vel_e(i),tpr_e(i),tpp_e(i),efield(i)
                    
           enddo
        close(201)



        ! Calculate electon temp using Titherodge equation if ele=0
        select case (eqe)
            case (1)
                call ele_fluid(dtg,num+1,fheate,curt,Te)
            case default
                CALL ele_den(num+1)
                CALL ele_vel(num+1,curt)

                call Titheridge_Te(Req,Te)
                tpr_e=Te
                tpp_e=Te
        end select

        print *, 'nst=', nst, 'rank=', rank, 'finished ele_fluid in GSK'
        !call MPI_BARRIER(MPI_COMM_WORLD,ierr)
        !pause

        !  XYWU mapping magnetospheric component 12/15/99
        if(nst.ge.HOTPON.and.nst.le.HOTPOF) then
          call Liouville(num,poten)

          den_hote=den_O+den_H+den_He+den_p-den_e
        else
          call init_array4()
        endif

        ! update electric field
        call efdfind(efield,poten)


        open(201,file='moments_move.dat',status='unknown')
           do i=1,num
              write(201,*) i,sg(i),(GLL(i)+GLL(i+1))*57.3/2.0,den_O(i),vel_O(i),&
                           tpr_O(i),tpp_O(i),den_H(i),vel_H(i),tpr_H(i),tpp_H(i),&
                           den_He(i),vel_He(i),tpr_He(i),tpp_He(i),den_e(i),&
                           vel_e(i),tpr_e(i),tpp_e(i),efield(i)
                    
           enddo
        close(201)

        print *, 'nst=', nst, 'rank=', rank, 'finished efdfind in GSK'
        !call MPI_BARRIER(MPI_COMM_WORLD,ierr)
        !pause

      enddo

        close(63)

     !write(334,'(2e16.7)') nst*dtg, energy
!****** output results ***********
      if(mod(nst,iout).eq.0) then

        !write(*,*) 'GSK output ...'

        !call write_ptcl(rank)

        print *, 'nst=', nst, 'rank=', rank, 'finished write ptcl in GSK'
     
        call openfiles(num,nst)
        ! output results (O+, H+ & e moments, efield, poten & wave heating rate)
        call print_moments(nst,efield,poten,curt,dppo,dpph,dpphe)

        print *, 'nst=', nst, 'rank=', rank, 'finished print moments in GSK'

        ! write ele moments. Add ion moments for qucik graphic display
           open(11,file='/home/wangy/DyFk/INITIAL/moments.dat', &
             status='unknown',form='formatted')
           write(11,102)
102        format('#',10x,'mlat',13x,'Ne',13x,'Te',11x,'Tepr',11x,'Tepp',12x, &
                 'Ve',12x,'efd',11x,'NO+',11x,'VO+',11x,'TOpr',11x,'TOpp', &
                 12x,'NH+',12x,'VH+',11x,'THpr',11x,'THpp',10x,'NHe+',10x, &
                 'VHe+',9x,'THepr+',9x,'THepp',10x,'qpr_O',10x,'qpp_O',10x, &
                 'qpr_H',10x,'qpp_H',9x,'qpr_He',9x,'qpp_He',10x,'qpr_e', &
                 10x,'qpp_H')
           do i=1,num
              write(11,103) (GLL(i)+GLL(i+1))*57.3/2.0,den_e(i),Te(i),tpr_e(i),tpp_e(i), &
                    vel_e(i),efield(i),den_O(i),vel_O(i), &
                    tpr_O(i),tpp_O(i),den_H(i), &
                    vel_H(i),tpr_H(i),tpp_H(i), &
                    den_He(i),vel_He(i),tpr_He(i),tpp_He(i), &
                    qpr_O(i),qpp_O(i),qpr_H(i), &
                    qpp_H(i),qpr_He(i),qpp_He(i), &
                    qpr_e(i),qpp_e(i)
           enddo
           i=num+1
             write(11,103) GLL(i)*57.3,den_e(i),Te(i),tpr_e(i),tpp_e(i), &
                    vel_e(i),efield(i),den_O(i),vel_O(i), &
                    tpr_O(i),tpp_O(i),den_H(i), &
                    vel_H(i),tpr_H(i),tpp_H(i), &
                    den_He(i),vel_He(i),tpr_He(i),tpp_He(i), &
                    qpr_O(i),qpp_O(i),qpr_H(i), &
                    qpp_H(i),qpr_He(i),qpp_He(i), &
                    qpr_e(i),qpp_e(i)
103        format(1x,27e15.6)
           close(11)

      print *, 'nst=', nst, 'rank=', rank, 'finished write INITIAL/moments in GSK'
		
      endif

!... boundary conditions for FLIP
      call flip_ub(nst)

      print *, 'nst=', nst, 'rank=', rank, 'finished flip_ub in GSK'

      call MPI_BARRIER(MPI_COMM_WORLD,ierr)
end subroutine tgsk
!----------------------------------------------------------------------
!  Ion heating by electrons Y.-J. Su on 10/2/95
!
!    Input:  v     - ion velocity (para or perp) (in Re/s)
!            feht  - ion heating rate by electrons (ergs/s)
!            m     - ion mass (kg)
!            dt    - time interval (s)
!            dene  - electron density (cm^-3)
!            Te    - electron temperature (K)
!            Ti    - ion (O+ or H+) temperature (K)
!            ns     - = 0 for parallel heating, = 1 for perp heating
!
!    Output: v - ion velocity (para or perp) after heating (Re/s)
!
!  Rewitten by Jiannan Tu, Feb 13, 2003. JNT 1/25/10 Rev
!----------------------------------------------------------------------
subroutine addheat(i,s,v,m,dt,dene,Te,dens,Ti,ns)

      use gskparm

      real(8) s,v,m,dt,w,theta,sig,dw,w1,dv,pi2,Re,gau
      real(8) dene(nc),Te(nc),dens(nc),Ti(nc)
      real(8) feht,aa,den,T_e,deni,T_i,ran2

      integer ns,i

      data pi2/6.283185308d0/,Re/6371.2d+3/

! first check if input feht < 0 and Te > Ti. If so, need to calculate
! heating rate feht (Banks & Kockarts, Aeronomy, pt B, p.263, 1973)
      aa=(s-sg(i))*ods(i)
      den=aa*(dene(i+1)-dene(i))+dene(i)
      deni=aa*(dens(i+1)-dens(i))+dens(i)
      T_e=aa*(Te(i+1)-Te(i))+Te(i)
      T_i=aa*(Ti(i+1)-Ti(i))+Ti(i)
      if(den.le.0.0d0.or.T_e.le.0.0d0.or.deni.le.0.0d0.or.T_i.le.0.0d0) return

      feht=7.7d-19*den*(T_e-T_i)/(T_e**1.5)
      if (feht.le.0.0d0) return

      sig=1.14d0*feht*dt
      dw=gau(sig)

      if(ns.eq.0) then
        dv=sqrt(2.0d-7*dw/m)/Re
        if (ran2(iseed).lt.0.5d0) dv=-dv
        v=v+dv
      else
        theta=pi2*ran2(iseed)
        w=0.5d7*m*(v*Re)**2
        w1=w+dw+2.*sqrt(w)*sqrt(dw)*cos(theta)
        v=sqrt(2.0d-7*w1/m)/Re
      endif

end subroutine addheat

!-------------------------------------------------------------------
!  Zero array elements of ion moments
!
!  Author:  Jiannan Tu
!  Date:    Feb 15, 2003
!-------------------------------------------------------------------  
subroutine init_array1()
      use gskparm

      integer numb

      numb=num+1

      den_O(1:numb)=0.0d0
      vel_O(1:numb)=0.0d0
      tpr_O(1:numb)=0.0d0
      tpp_O(1:numb)=0.0d0
      qpr_O(1:numb)=0.0d0
      qpp_O(1:numb)=0.0d0
      den_H(1:numb)=0.0d0
      vel_H(1:numb)=0.0d0
      tpr_H(1:numb)=0.0d0
      tpp_H(1:numb)=0.0d0
      qpr_H(1:numb)=0.0d0
      qpp_H(1:numb)=0.0d0
      den_He(1:numb)=0.0d0
      vel_He(1:numb)=0.0d0
      tpr_He(1:numb)=0.0d0
      tpp_He(1:numb)=0.0d0
      qpr_He(1:numb)=0.0d0
      qpp_He(1:numb)=0.0d0

end subroutine init_array1

!-------------------------------------------------------------------
!  Zero array elements of accumulated ion moments
!
!  Author:  Jiannan Tu
!  Date:    March 17, 2003
!-------------------------------------------------------------------  
subroutine init_array2(denO,velO,tprO,tppO,qprO,qppO,denH,velH, &
       tprH,tppH,qprH,qppH,denHe,velHe,tprHe,tppHe,qprHe,qppHe)

      use gskparm

      integer numb

      real(8) denO(nc),velO(nc),tprO(nc),tppO(nc),qprO(nc),qppO(nc)
      real(8) denH(nc),velH(nc),tprH(nc),tppH(nc),qprH(nc),qppH(nc)
      real(8) denHe(nc),velHe(nc),tprHe(nc),tppHe(nc)
      real(8) qprHe(nc),qppHe(nc)

      numb=num+1

      denO(1:numb)=0.0d0
      velO(1:numb)=0.0d0
      tprO(1:numb)=0.0d0
      tppO(1:numb)=0.0d0
      qprO(1:numb)=0.0d0
      qppO(1:numb)=0.0d0
      denH(1:numb)=0.0d0
      velH(1:numb)=0.0d0
      tprH(1:numb)=0.0d0
      tppH(1:numb)=0.0d0
      qprH(1:numb)=0.0d0
      qppH(1:numb)=0.0d0
      denHe(1:numb)=0.0d0
      velHe(1:numb)=0.0d0
      tprHe(1:numb)=0.0d0
      tppHe(1:numb)=0.0d0
      qprHe(1:numb)=0.0d0
      qppHe(1:numb)=0.0d0

end subroutine init_array2

!-------------------------------------------------------------------
!  Zero array elements of ion velocity distributions
!
!  Author:  Jiannan Tu
!  Date:    Feb 15, 2003
!-------------------------------------------------------------------  
subroutine init_array3(vsvp_O,vss_O,vps_O,vsvp_H,vss_H,vps_H, &
                       vsvp_He,vss_He,vps_He)

      use gskparm

      real(8) vss_O(noss,100),vps_O(nosp,100),vsvp_O(nossm,nospm)
      real(8) vss_H(nhss,100),vps_H(nhsp,100),vsvp_H(nhssm,nhspm)
      real(8) vss_He(nhess,100),vps_He(nhesp,100),vsvp_He(nhessm,nhespm)

      vsvp_O=0.0d0
      vsvp_H=0.0d0
      vsvp_He=0.0d0
      vss_O=0.0d0
      vps_O=0.0d0
      vss_H=0.0d0
      vps_H=0.0d0
      vss_He=0.0d0
      vps_He=0.0d0

end subroutine init_array3

!---------------------------------------------------------------------
!  Zero array elements of ion magnetospheric plasma moments
!
!  Author:  Jiannan Tu
!  Date:    March 25, 2003
!--------------------------------------------------------------------- 
subroutine init_array4()

      use gskparm

      integer numb

      numb=num+1

      den_p(1:numb)=0.0d0
      den_hote(1:numb)=0.0d0
      tpp_p(1:numb)=0.0d0
      tpr_p(1:numb)=0.0d0
      tpp_hote(1:numb)=0.0d0
      tpr_hote(1:numb)=0.0d0
     
end subroutine init_array4

!----------------------------------------------------------
!... Smooth O+ and H+ density & velocity profiles
!----------------------------------------------------------
subroutine smooth10(n)

        use gskparm

        integer n,j

!... smooth ten times
        do j=1,10
          !call smoothp(den_O,n)
          !call smoothp(den_H,n)
          call smoothp(vel_O,n)
          call smoothp(vel_H,n)
          !call smoothp(den_He,n)
          call smoothp(vel_He,n)
          call smoothp(vel_e,n)
          call smoothp(tpr_O,n)
          call smoothp(tpp_O,n)
          call smoothp(tpr_H,n)
          call smoothp(tpp_H,n)
          call smoothp(tpr_He,n)
          call smoothp(tpp_He,n)
        enddo

end subroutine smooth10
